/*
 *  Licensed to the Apache Softwimport org.apache.commons.lang.StringUtils;

import fr.loof.fonzie.JPQLPersistenceQuery;
file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.parsers;

import java.lang.reflect.Field;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.springframework.util.ReflectionUtils;


public abstract class KeyWord
    implements Comparable<KeyWord>
{
    private String keyword;

    public KeyWord( String keyWord )
    {
        this.keyword = keyWord;
    }

    public KeyWord()
    {
        this.keyword = getClass().getSimpleName();
    }

    public int compareTo( KeyWord o )
    {
        // longer first
        return o.keyword.length() - keyword.length();
    }

    public final boolean matches( JPQLPersistenceQuery query, Class entity, String constraint, List<Class> types )
    {
        if ( constraint.endsWith( keyword ) )
        {
            String property = StringUtils.uncapitalize( StringUtils.removeEnd( constraint, keyword ) );
            Class propertyType = checkFieldExists( entity, property );

            int consumed = process( query, property, propertyType );
            while ( consumed-- > 0 )
            {
                Class arg = types.remove( 0 );
                checkParameterType( entity, property, propertyType, arg );
            }
            return true;
        }
        return false;
    }

    protected void checkParameterType( Class entity, String property, Class propertyType, Class arg )
    {
        if ( !arg.isAssignableFrom( propertyType ) )
        {
            throw new IllegalStateException( "Constraint on " + entity.getSimpleName() + "." + property + " ("
                + propertyType.getSimpleName() + ") is not compatible with parameter type " + arg );
        }
    }

    public static Class checkFieldExists( Class entity, String property )
    {
        Class clazz = entity;
        Field attribute = ReflectionUtils.findField( entity, property );
        if ( attribute == null )
        {
            throw new IllegalStateException( "Entity " + entity.getSimpleName() + " has no attribute " + property );
        }
        return attribute.getType();
    }

    protected abstract int process( JPQLPersistenceQuery query, String property, Class propertyType );
}